#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #####################################################################
# Copyright (c) 2005 Jiří Popek <jiri.popek@gmail.com>
#
# name        : pDict
# description : Python and Qt dictionary search
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# LICENCE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# author : Jiří Popek
# email  : jiri.popek@gmail.com
# date   : 26.1.2005
#
# last modify: 12.7.05
# history:
#       12.7.05 multiple datapaths
#       10.7.05 added mutli-gui behaviour
# #####################################################################

from ConfigParser import ConfigParser
import os
import sys
import StringIO
import locale

DEBUG=False

APP_NAME='pDict'
APP_VERSION='1.5.1'

DATA_PATH = [
            os.path.expanduser('~/.pdict'), # primary
            #os.path.expanduser('~/pdict'),  # old primary
            os.path.join('%s' % sys.prefix, 'share', 'pdict')
            ]

DICT_FILE_SUFFIX  = '.txt'
INDEX_FILE_SUFFIX = '.idx'
ICON_FILE_SUFFIX  = '.png'
DESC_FILE_SUFFIX  = '.desc'

config_defaults =(
            "[global]\n"
            "GUI=qt\n"
            "default_dict=0\n"
            "on_the_fly=0\n"
            "[Main form]\n"
            "x=0\n"
            "y=0\n"
            "width=480\n"
            "height=320\n"
            "[Description dialog]\n"
            "x=0\n"
            "y=0\n"
            "width=420\n"
            "height=160\n"
            )

config_files = [
    os.path.abspath('pdictrc'), # current dir
    os.path.expanduser('~/.pdict/pdictrc'), # user's home
    ]

DEPTH = 3 # how many chars are used for indexing

TAB     = "\t"
LF      = "\n"
CRLF    = "\r\n"

class pDictError(Exception):
    def __init__(self, message):
        self.message = message

# class for config file parsing
class Config(ConfigParser):
    def __init__(self):
        ConfigParser.__init__(self)
        self.fromstring(config_defaults)
        self.read(config_files)
    
    def fromstring(self, string):
        """load configfile from string (default)"""
        sio = StringIO.StringIO(string)
        self.readfp(sio)
    
    def get(self, section, key):
        val = ConfigParser.get(self, section, key)
        
        try:
            val = eval(str(val))
        except:
            pass
        return val
    
    def save(self):
        fp = open(os.path.expanduser('~/.pdict/pdictrc'), 'w')
        self.write(fp)
        fp.close()

pDictConfig = Config()

# format transation to html code when is to be showed within description dialog
def formatDesc(item):
    """format descriptions to show in descriptions window"""
    txt =  ('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" "http://www.w3.org/TR/html4/loose.dtd">'
            '<html>'
            '<head>'
            '   <meta http-equiv="Content-Type" content="text/html; charset=utf-8">'
            '</head>'
            '<body>')
    
    key, val, meta = item
    txt += '<h2>' + key + '</h2>'
    
    if len(meta):
        txt += '<br><i>' + meta[0] + '</i>'
    if val != '':
        txt += '<br><br>' + val
    # under hrule
    tt = ''
    
    if len(meta):
        for v in meta[1:]:
            if v != '':
                tt += v + ', '
        tt = tt[:-2]
    
    if tt != u'':
        txt += '<br>' + '<hr>' + tt
    
    txt += ('</body>'
            '</html>')
    
    return txt
