#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #####################################################################
# Copyright (c) 2005 Jiří Popek <jiri.popek@gmail.com>
#
# name        : pDict
# description : dictionary search
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# LICENCE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# author : Jiří Popek
# email  : jiri.popek@gmail.com
# date   : 15.7.2005
#
# last modify: 1.8.05
# history:
#        1.8.05 added dictionary swapping
#       15.7.05 initial version
# #####################################################################

import gtk
import gobject
import gtkhtml2

from globals import *
from dict import *
import updater
from threading import Timer
import Queue
from time import sleep
import cPickle

__author__ = 'Jiří Popek'
__doc__ = """GTK dependent definitions"""

ENTER_KEY_CODE  = 65293
ESC_KEY_CODE    = 65307
R_KEY_CODE      = 114

queue = None

########################################################################
## message boxes
class pDictMessageBox(gtk.MessageDialog):
    def __init__(self, parent=None):
        gtk.MessageDialog.__init__(self, parent)
    
    def information(self, title, text):
        self.information(title, text)

########################################################################
## main applications class
class pDictApplication:
    def __init__(self, argv):
        pass
    def setMainWidget(self, main):
        pass
    def exec_loop(self):
        # !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        # have to call this explicitly before gtk.main() if using threads!
        # if not, extern thread will not comunicate properly with
        # gtk objects (thread doesn't works until its end (freezes))
        # Sometimes freezes unexpectedly within main loop and does not
        # works until its end - does not append all translations and does
        # not call endFunctionCall (endFuncCallBack)
        gtk.gdk.threads_init()
        gtk.main()
        return 0

########################################################################
## base class for all windows (save/load geometry stuff)
class MyWindow:
    def saveGeometry(self, config, section_name):
        # GTK does not return proper x, y values when window is hidden
        self.show()
        x, y = self.get_position()
        config.set(section_name, 'x', x)
        config.set(section_name, 'y', y)
        width, height = self.get_size()
        config.set(section_name, 'width', width)
        config.set(section_name, 'height', height)
    
    def loadGeometry(self, config, section_name):
        g = [config.get(section_name, key) for key in ('x', 'y', 'width', 'height')]
        self.move(g[0], g[1])
        self.set_geometry_hints(base_width=g[2], base_height=g[3])
        self.resize(g[2], g[3])

########################################################################
## List view class
class DescrListView(gtk.ScrolledWindow):
    """list view with found items"""
    def __init__(self, parent):
        gtk.ScrolledWindow.__init__(self)
        self.main_wnd = parent
        
        self.liststore = gtk.ListStore(str, str, object)
        self.treeview = gtk.TreeView(self.liststore)
        # create the TreeViewColumns to display the data
        self.treeview.columns = 2*[None]
        self.treeview.columns[0] = gtk.TreeViewColumn(_('expression'))
        self.treeview.columns[1] = gtk.TreeViewColumn(_('translation'))
        
        for n in range(2):
            # add columns to treeview
            self.treeview.append_column(self.treeview.columns[n])
            self.treeview.columns[n].set_resizable(True)
            # create a CellRenderers to render the data
            self.treeview.columns[n].cell = gtk.CellRendererText()
            self.treeview.columns[n].pack_start(self.treeview.columns[n].cell, False)
            # set the cell attributes to the appropriate liststore column
            self.treeview.columns[n].set_attributes(self.treeview.columns[n].cell, text=n)
        
        # make treeview searchable
        self.treeview.set_search_column(0)
        self.treeview.set_rules_hint(True)
        #self.set_headers_visible(False)
        
        #self.set_size_request(400, 300)
        self.add(self.treeview)
        self.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.treeview.show()
        
        selection = self.treeview.get_selection()
        selection.set_mode(gtk.SELECTION_SINGLE)
        selection.connect("changed", self.on_selection_changed)
    
    def on_selection_changed(self, selection):
        """ mouse button press event """
        model, iter = selection.get_selected()
        item = model.get(iter, 0, 1, 2)
        if item:
            self._showDesc(item)
    
    def processIncoming(self):
        """ add new data funct - adds few items-20 (will be called again later) """
        i = 20
        while(i):
            try:
                msg = queue.get(0)
                l = len(msg)
                if l>1:
                    k, v = msg[0], msg[1]
                else:
                    k, v = msg[0], ''
                lvi = self.liststore.append([k, v, msg[2:]])
                i -= 1
            except Queue.Empty:
                return False # do not call again
        
        return True # call again
    
    def _showDesc(self, item):
        self.main_wnd.desc_dlg.setItem(item)
        self.main_wnd.desc_dlg.show_now()
    
    def clear(self):
        self.liststore.clear()

########################################################################
## descripting dialog (show when clicking on listview item)
class DescrDialog(gtk.Window, MyWindow):
    def __init__(self):
        gtk.Window.__init__(self)
        self.vw = gtkhtml2.View()
        self.add(self.vw)
        self.show_all()
        
        self.doc = gtkhtml2.Document()
        self.vw.set_document(self.doc)
        
        self.connect("key-press-event", self.keyPressEvent)
        self.connect("delete_event", self.closeEvent)
    
    def setItem(self, item):
        self.set_title(item[0])
        
        html =  unicode(formatDesc(item))
        self.doc.open_stream('text/html')
        self.doc.write_stream(html)
        self.doc.close_stream()
    
    def closeEvent(self, widget, event):
        self.hide()
        return True
    
    def keyPressEvent(self, widget, event):
        keyval = event.keyval
        # clear Entry box when ESC pressed
        if keyval==ESC_KEY_CODE or  keyval==ENTER_KEY_CODE:
            self.hide()

########################################################################
## main window class
class pDictForm(gtk.Window, MyWindow):
    index = {}
    def __init__(self, name):
        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
        self.searching = False
        self.set_title(name)
        # vertical box - root
        vbox = gtk.VBox()
        self.add(vbox)
        hbox = gtk.HBox()
        vbox.pack_start(hbox, False)
        # text input control
        self.lineEditSource = gtk.Entry()
        hbox.pack_start(self.lineEditSource, True, True, 0)
        self.lineEditSource.connect('activate', self.leTxtChangedFromSLOT, True)
        self.lineEditSource.connect('changed', self.leTxtChangedFromSLOT, pDictConfig.get('global', 'on_the_fly'))
        # combo box
        self.cbDicts = gtk.ListStore(gobject.TYPE_STRING)
        self.cbDictsBox = gtk.ComboBox(self.cbDicts)
        cell = gtk.CellRendererText()
        self.cbDictsBox.pack_start(cell, True)
        self.cbDictsBox.add_attribute(cell, 'text', 0)
        hbox.pack_start(self.cbDictsBox, False, False, 0)
        self.cbDictsBox.connect('changed', self.ComboDictsClickedSLOT)
        
        mgr = updater.Updater()
        self.dicts = mgr.getAvailableDictionaries()
        if len(self.dicts)==0:
            raise pDictError(_('err.there_is_not_any_dictionary'))
        
        # insert all dictionary names to Combobox
        for path, dct in self.dicts.iteritems():
            txt = dct['LABEL']
            listiter = self.cbDicts.append()
            self.cbDicts.set(listiter, 0, dct['LABEL'])
        
        self.listViewChoice = DescrListView(self)
        vbox.pack_start(self.listViewChoice)
        
        self.sb = gtk.Statusbar()
        self.sb.context_id = self.sb.get_context_id('message')
        vbox.pack_start(self.sb, False)
        
        self.desc_dlg = DescrDialog() # descriptions
        self.desc_dlg.set_transient_for(self)
        self.desc_dlg.hide()
        
        self.connect("delete_event", self.closeEvent)
        self.show_all()
        self.wt = None
        
        self.ds = None
        self.source_id = None
        self.connect("key-press-event", self.keyPressEvent)
    
    def on_swap_langs(self):
        """swap translate direction"""
        a, b = os.path.splitext(os.path.basename(self.datafile))[0].split('-')
        sd = b + '-' + a
        
        i=0
        for dn,v in self.dicts.iteritems():
            if v['DICTIONARY_NAME']==sd:
                self.setCurrentItem(i)
                self.leTxtChangedFromSLOT(self.lineEditSource,  pDictConfig.get('global', 'on_the_fly'))
            i+=1
    
    def ComboDictsClickedSLOT(self, txt):
        """Combo box item changed slot"""
        id = self.cbDictsBox.get_active()
        if self.setDict(id)==True:
            self.leTxtChangedFromSLOT(self.lineEditSource, False)
    
    def leTxtChangedFromSLOT(self, entry, otf):
        """expression changed slot-restart searching"""
        text = entry.get_text()
        if text=='':
            return
        
        if otf:
            self.startSearch(text)
    
    def startSearch(self, text):
        """starts searching thread"""
        global queue
        # stop running search thread
        self.stopThread()
        
        if self.source_id:
            gobject.source_remove(self.source_id)
        
        ln = len(text) # word length
        self.listViewChoice.clear()
        self.sb.push(self.sb.context_id, _('searching...'))
        
        uni = unicode(text)
        # empty the que
        queue = Queue.Queue()
        # start new thread
        self.ds = DictionarySearch([uni, ln], queue, self.endFuncCallBack, self.datafile, self.index) # search engine
        # call function for appending periodically
        # call function self.periodicCall after 0.1s to add items to listbox
        self.source_id = gobject.timeout_add(100, self.periodicCall)
    
    def periodicCall(self):
        """function to call periodicaly - add few new item to listbox"""
        if not self.ds: return # searching thread haven't been started yet
        if self.ds.isAlive()==True: 
            # searching thread is running - adding items
            # even if queue is empty
            self.listViewChoice.processIncoming()
        else:
            # searching thread finished, add items until queue is empty
            ret = self.listViewChoice.processIncoming()
            return ret
    
    def setDict(self, dict_idx):
        """loads new disctionary index file
            params:
                dict_idx - index of dictionary (taken from self.dicts
        """
        try:
            fb = self.dicts.keys()[dict_idx]
            data_file = fb + DICT_FILE_SUFFIX
            index_file = fb + INDEX_FILE_SUFFIX
            icon_file = fb + ICON_FILE_SUFFIX
            # load dictionary index from cPicke file
            f = open(index_file, 'r')
            val = cPickle.load(f)
            f.close()
            # set data filename and loaded index to instance variables
            self.datafile = data_file
            self.index    = val
            # define default icon filename
            px = os.path.join('%s', 'share', 'pdict', 'default_dict.png')
            if os.path.exists(icon_file): # check if exists current dictionary icon
                px = icon_file
            # load the icon
            self.set_icon_from_file(px)
            self.desc_dlg.set_icon_from_file(px)
            return True
        except IndexError:
            return False
    
    def keyPressEvent(self, widget, event):
        """key press event slot"""
        keyval = event.keyval
        # clear Entry box when ESC pressed
        if keyval==ESC_KEY_CODE:
            self.stopThread()
            self.lineEditSource.set_text('')
        # start searching when Enter pressed
        elif keyval==ENTER_KEY_CODE:
            self.startSearch(self.lineEditSource.get_text())
        # CTRL+R pressed - swap direction of translation
        elif keyval==R_KEY_CODE and event.state==gtk.gdk.CONTROL_MASK:
            self.on_swap_langs()
    
    def getCurrentItem(self):
        """get index of current dictionary"""
        return self.cbDictsBox.get_active()
    def setCurrentItem(self, idx):
        """set current Combobox item (current dictionary)"""
        cnt = len(self.cbDicts)
        if cnt>0:
            if idx>=cnt:
                idx=0
            
            self.cbDictsBox.set_active(idx)
            self.ComboDictsClickedSLOT(idx)
        else:
            pDictMessageBox().information(self, _('Error'), _('Could not find any dictionary file'))
            return False
        
        return True
    
    def endFuncCallBack(self, cnt):
        """this is called by searching when finishing"""
        txt = _('found') + ' ' + str(cnt) + ' ' + _('records')
        # emit text-pushing Event
        self.sb.emit('text_pushed', self.sb.context_id, txt)
    
    def stopThread(self):
        """proper stopping searching"""
        if self.ds:
            self.ds.shutdown()
            self.ds.join() # wait for thread to finish
            self.ds = None
    
    def closeEvent(self, widget, arg):
        """main window closing"""
        self.stopThread() # end searching thread
        gtk.main_quit()
        return True
    
    def show(self):
        gtk.Window.show(self)
