#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #####################################################################
# Copyright (c) 2005 Jiří Popek <jiri.popek@gmail.com>
#
# name        : pDict
# description : dictionary search
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# LICENCE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# author : Jiří Popek
# email  : jiri.popek@gmail.com
# date   : 23.3.2005
#
# last modify: 15.7.05
# history:
#        1.8.05 added dictionary swapping
#       12.7.05 multiple datapaths
#       10.7.05 added mutli-gui behaviour
# #####################################################################

from qt import QListViewItem, QStatusBar, SIGNAL, QTextBrowser, QCustomEvent, \
        QString, QDialog, QListView, QVBoxLayout, QHBoxLayout, QLineEdit, \
        QRect, QPixmap, QGridLayout, QApplication, QObject, QCheckBox, QTimer, \
        QMainWindow, QKeyEvent, QPoint, QComboBox, QMessageBox, QEvent, QWidget

__author__ = u'Jiří Popek'
__doc__ = """Qt toolkit dependent definitions"""

from globals import *
from dict import *
import cPickle
import updater
import Queue
import gettext

ENTER_KEY_CODE  =4100
CTRL_KEY_CODE   = 512
R_KEY_CODE      = 82

queue = None

def _(txt):
	return unicode(gettext.gettext(txt), 'utf-8')

########################################################################
## message boxes
class pDictMessageBox(QMessageBox):
    def __init__(self, parent=None):
        QMessageBox.__init__(self, parent)
    
    def information(self, title, text):
        self.information(title, text)

########################################################################
## main applications class
class pDictApplication(QApplication):
    def __init__(self, args):
        QApplication.__init__(self, args)
    def setMainWidget(self, main):
        QApplication.setMainWidget(self, main)
    def exec_loop(self):
        return QApplication.exec_loop(self)

########################################################################
## base class for all windows (save/load geometry stuff)
class MyWindow:
    def saveGeometry(self, config, section_name):
        config.set(section_name, 'x', self.x())
        config.set(section_name, 'y', self.y())
        config.set(section_name, 'width', self.width())
        config.set(section_name, 'height', self.height())
        
    def loadGeometry(self, config, section_name):
        g = [config.get(section_name, key) for key in ('x', 'y', 'width', 'height')]
        geom = QRect(g[0], g[1], g[2], g[3])
        self.setGeometry(geom)

########################################################################
## event for item adding to list
class NewItemEvent( QCustomEvent ):
    """ event raised when new item is added """
    def __init__( self, arg ):
        QCustomEvent.__init__(self, QCustomEvent.User)
        self.setData( arg )

########################################################################
## List view class
class DescrListView(QListView):
    """ list view with found items """
    def __init__(self, parent, name=None):
        QListView.__init__(self, parent, name)
        self.parent = parent
        
        self.addColumn("word")
        self.addColumn("sep", 10)
        self.addColumn("translation")
        self.header().hide()
        self.setAllColumnsShowFocus(1)
        self.setResizeMode(QListView.LastColumn)
        self.setMidLineWidth(50)
        self.setSorting(0, True)
    
    def wheelEvent(self, e):
        """ mouse wheel event """
        QListView.wheelEvent(self, e)
        self.parent.lineEditSource.setFocus()
    
    def keyPressEvent(self, e):
        """ key press event """
        k = e.key()
        if k==ENTER_KEY_CODE:
            item = self.selectedItem()
            if item:
                self._showDesc(item)
        
        QListView.keyPressEvent(self, e)
    
    def processIncoming(self):
        """ add new data event """
        i = 20
        while(i):
            try:
                msg = queue.get(0)
                l = len(msg)
                
                if l>1:
                    k, v = msg[0], msg[1]
                else:
                    k, v = msg[0], ''
                lvi = QListViewItem(self, k, '', v)
                lvi.meta = msg[2:]
                i -= 1
            except Queue.Empty:
##                # highlight the first item
##                first = self.firstChild()
##                self.setSelected(first, True)
##                self.setCurrentItem(first)
                return False
        return True
    
    def contentsMousePressEvent(self, e) :
        QListView.contentsMousePressEvent(self, e)
        p = QPoint( self.contentsToViewport(e.pos()) )
        item = self.itemAt(p)
        
        if item == None:
            return
        
        self._showDesc(item)
    
    def _showDesc(self, item):
        self.parent.desc_dlg.setItem(item)
        self.parent.desc_dlg.show()
        self.parent.setActiveWindow()
        self.parent.lineEditSource.setFocus()

########################################################################
## descripting dialog (show when clicking on listview item)
class DescrDialog(QTextBrowser, MyWindow):
    def __init__(self, parent):
        QTextBrowser.__init__(self)
    
    def setItem(self, lvi):
        self.setCaption(lvi.text(0))
        tf = [unicode(lvi.text(0)), unicode(lvi.text(2)), lvi.meta]
        html = formatDesc(tf)
        self.setText(html)
    
    def keyPressEvent ( self, e ):
        if e.key()==QKeyEvent.Key_Escape or e.key()==4100:
            self.hide()

########################################################################
## main window class
class pDictForm(QWidget, MyWindow):
    index = {}
    def __init__(self, name):
        QWidget.__init__(self, None, name)
        self.setCaption(name)
        
        self.vb0 = QVBoxLayout(self, 3)
        self.hb0 = QHBoxLayout(self.vb0)
        
        self.lineEditSource = QLineEdit(self)
        self.hb0.addWidget(self.lineEditSource)
        self.cbDicts = QComboBox(self)
        self.hb0.addWidget(self.cbDicts)
        
        mgr = updater.Updater()
        self.dicts = mgr.getAvailableDictionaries()
        
        if len(self.dicts)==0:
            raise pDictError(_('Could not find any dictionary file'))
        
        # insert all dictionary names to Combobox
        for path, dct in self.dicts.iteritems():
            self.cbDicts.insertItem(dct['LABEL'])
        # list view
        self.listViewChoice = DescrListView(self)
        self.vb0.addWidget(self.listViewChoice)
        # status bar
        self.sb = QStatusBar(self)
        self.vb0.addWidget(self.sb)
        
        self.ds = None
        self.desc_dlg = DescrDialog(self) # descriptions
        
        self.connect(self.lineEditSource, SIGNAL("textChanged(const QString&)"), self.leTxtChangedFromSLOT)
        self.connect(self.cbDicts, SIGNAL("activated(int)"), self.ComboDictsClickedSLOT)
    
    def on_swap_langs(self):
        """swap translate direction"""
        a, b = os.path.splitext(os.path.basename(self.datafile))[0].split('-')
        sd = b + '-' + a
        
        i=0
        for dn,v in self.dicts.iteritems():
            if v['DICTIONARY_NAME']==sd:
                self.setCurrentItem(i)
                self.leTxtChangedFromSLOT(self.lineEditSource)
            i+=1
    
    def ComboDictsClickedSLOT(self, id):
        """Combo box item changed slot"""
        if self.setDict(id) ==  True:
            self.leTxtChangedFromSLOT(self.lineEditSource.text())
    
    def leTxtChangedFromSLOT(self, text):
        """expression changed slot-restart searching"""
        if text == '':
            return
        if pDictConfig.get('global', 'on_the_fly'):
            self.startSearch(text)
    
    def startSearch(self, text):
        """starts searching thread"""
        global queue
        # stop running search thread
        self.stopThread()
        
        ln = len(text) # word length
        self.listViewChoice.clear()
        self.listViewChoice.setColumnWidth(0, 10)
        self.sb.message(_('searching...'))
        
        uni = unicode(text)
        # empty the que
        queue = Queue.Queue()
        # start new thread
        self.ds = DictionarySearch([uni, ln], queue, self.endFuncCallBack, self.datafile, self.index) # search engine
        # call function self.periodicCall after 0.2s to add items to listbox
        QTimer.singleShot(0.2, self.periodicCall)
    
    def periodicCall(self):
        """function to call periodicaly - add few new item to listbox"""
        if not self.ds: return # searching thread haven't been started yet
        if self.ds.isAlive()==True: 
            # searching thread is running - adding items
            # even if queue is empty
            self.listViewChoice.processIncoming()
        else:
            # searching thread finished, add items until queue is empty
            ret = self.listViewChoice.processIncoming()
            if ret==False: # end
                return
        # call this function again after 0.2s to add new items to listbox
        QTimer.singleShot(0.2, self.periodicCall)
    
    def setDict(self, dict_idx):
        """loads new disctionary index file
            params:
                dict_idx - index of dictionary (taken from self.dicts
        """
        try:
            fb = self.dicts.keys()[dict_idx]
            data_file = fb + DICT_FILE_SUFFIX
            index_file = fb + INDEX_FILE_SUFFIX
            icon_file = fb + ICON_FILE_SUFFIX
            # load dictionary index from cPicke file
            f = open(index_file, 'r')
            val = cPickle.load(f)
            f.close()
            # set data filename and loaded index to instance variables
            self.datafile = data_file
            self.index    = val
            # define default icon filename
            df = os.path.join('%s', 'share', 'pdict', 'default_dict.png')
            if os.path.exists(icon_file): # check if exists current dictionary icon
                px = icon_file
            # load the icon
            px = QPixmap(px)
            self.setIcon(px)
            self.desc_dlg.setIcon(px)
            return True
        except IndexError:
            return False
    
    def keyPressEvent(self, e):
        """key press event slot"""
        key = e.key()
        state = e.state()
        if key==QKeyEvent.Key_Escape:
            self.stopThread()
            self.lineEditSource.setText('')
        elif key==ENTER_KEY_CODE:
            self.startSearch(self.lineEditSource.text())
        # CTRL+R pressed - swap direction of translation
        elif key==R_KEY_CODE and state==CTRL_KEY_CODE: # r and CTRL keys
            self.on_swap_langs()
        self.listViewChoice.keyPressEvent(e)
    
    def getCurrentItem(self):
        """get index of current dictionary"""
        return self.cbDicts.currentItem()
    def setCurrentItem(self, idx):
        """set current Combobox item (current dictionary)"""
        cnt = self.cbDicts.count()
        if cnt>0:
            if idx>=cnt:
                idx=0
            
            self.cbDicts.setCurrentItem(idx)
            self.ComboDictsClickedSLOT(idx)
        else:
            pDictMessageBox().information(self, _('Error'), _('Could not find any dictionary file'))
            return False
        
        return True
    
    def endFuncCallBack(self, cnt):
        """this is called by searching when finishing"""
        txt = _('found') + ' ' + str(cnt) + ' ' + _('records')
        self.sb.message(txt)
    
    def stopThread(self):
        """proper stopping searching"""
        if self.ds:
            self.ds.shutdown()
            self.ds.join() # wait for thread to finish
            self.ds = None
    
    def closeEvent(self, e):
        """main window closing"""
        self.stopThread()      # end searching thread
        self.desc_dlg.close()
        e.accept()             # accepting the close
