#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #####################################################################
# Copyright (c) 2005 Jiří Popek <jiri.popek@gmail.com>
#
# name        : pDict
# description : Python and Qt dictionary search
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# LICENCE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# author : Jiří Popek
# email  : jiri.popek@gmail.com
# date   : 26.1.2005
#
# last modify: 3.8.05
# history:
#        3.8.05 rewrited to use gettext
#       12.7.05 multiple datapaths
#       10.7.05 added mutli-gui behaviour
# #####################################################################

__author__ = u'Jiří Popek'
__doc__ = """main module"""

from sys import exit, argv
import os
from globals import *
import locale, gettext
import string

APP_NAME = 'pdict'
DIR = 'po'

# install gettext
pth = gettext.find(APP_NAME)
if pth:
    p = string.find(pth, 'locale')
    pth = pth[:p+len('locale')]
else:
    pth = DIR

gettext.bindtextdomain(APP_NAME, pth)
gettext.textdomain(APP_NAME)
gettext.install(APP_NAME, pth, unicode=True)

gui = pDictConfig.get('global', 'GUI')

if gui=='qt':
    try:
        import qt
        from mainform_qt import *
    except:
        raise pDictError(_('pyQt is not installed!'))
elif gui=='gtk':
    try:
        import gtk
        import gtk.glade
        from mainform_gtk import *
        gtk.glade.bindtextdomain(APP_NAME, pth)
    except:
        raise pDictError(_('pyGTK is not installed'))
elif gui=='wx':
    try:
        import wxPython.wx
        from mainform_wx import *
    except:
        raise pDictError(_('wxPython is not installed!'))
else:
    raise pDictError(_('Do not know GUI framework') + ': %s' % gui)


def main():
    """main function - here everything begins (and ends...)"""
    a = pDictApplication(argv)
    main_form = pDictForm(APP_NAME + ' ' + APP_VERSION)
    # load geometries
    main_form.loadGeometry(pDictConfig, 'Main form')
    main_form.desc_dlg.loadGeometry(pDictConfig, 'Description dialog')
    # set current dictionary (in ComboBox)
    default_dict_idx = pDictConfig.get('global', 'default_dict')
    ld = main_form.setCurrentItem(default_dict_idx)
    if not ld:
        exit(1)
    
    a.setMainWidget(main_form)
    main_form.show()
    ret = a.exec_loop()
    
    # save params after close
    main_form.saveGeometry(pDictConfig, 'Main form')
    main_form.desc_dlg.saveGeometry(pDictConfig, 'Description dialog')
    ci = main_form.getCurrentItem()
    pDictConfig.set('global', 'default_dict', ci)
    # update config file
    pDictConfig.save()
    
    exit(ret)

if __name__ == "__main__":
    main()
