#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #####################################################################
# Copyright (c) 2005 Jiří Popek <jiri.popek@gmail.com>
#
# name        : pDict
# description : dictionary search
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# LICENCE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# author : Jiří Popek
# email  : jiri.popek@gmail.com
# date   : 12.7.2005
#
# last modify: 17.7.05
# history:
#       17.7.05 handle instalation to non-standard directory 
#               by Petr Vanek (petr@yarpen.cz)
# #####################################################################

"""Setup script for pDict"""

from distutils_locale_ext import setup
from distutils.file_util import copy_file
from globals import APP_VERSION as version
import glob
import os
import sys
import gettext

app_name='pdict'

# We need to parse the installation arguments before installation
# itself due the start_script modification.
# Variable realPrefix has to be set as the user specify. Resp.
# standard prython setup.py install is every time the same but
# user specific installation forced to modify the shell with
# prefix. Usage: python setup.py install --prefix=/home/user/pdict.
import distutils.dist
prepareDist = distutils.dist.Distribution()
if (not prepareDist.parse_command_line()):
    print "Error parsing arguments!"
    sys.exit(1)
#prepareDist.dump_option_dicts() #debug
installCommands = prepareDist.get_option_dict('install')
realPrefix = sys.prefix
# what to do?
if (installCommands.has_key('home')):
    print "There is a bug in distutils in --home option\nPlease, use --prefix instead.\n"
    sys.exit(1)
if (installCommands.has_key('prefix')):
    realPrefix = installCommands['prefix'][1]
    print 'installCommands:', str(installCommands)
    print "Installation in non-standard directory: %s\n" % realPrefix


long_desc = '''\
'''

classifiers = """\
Development Status :: 5 - Production/Stable
License :: OSI Approved :: GNU General Public License (GPL)
Programming Language :: Python
"""
# get python short version
py_short_version = '%s.%s' % sys.version_info[:2]

python_site_app_dir = os.path.join(realPrefix, 'lib', 'python%s' % py_short_version, 'site-packages', app_name)
if realPrefix!=sys.prefix:
    python_site_app_dir = realPrefix
    
# create startup script
start_script = \
"#!/bin/sh\n\
cd %s\n\
exec %s ./pdict.py $@" % (python_site_app_dir, sys.executable)
# write script
f = open('pdict', 'w')
f.write(start_script)
f.close()

def data_files():
    """Build list of data files to be installed"""
    files = [
            (os.path.join('share', 'pdict'), ['AUTHORS', 'COPYING', 'README']),
            (os.path.join('share', 'pdict'), ['default_dict.png']),
            (os.path.join('share', 'icons', 'hicolor', '64x64', 'apps'), ['pdict.png']),
            (python_site_app_dir, ['pdictrc']),
            ('bin', ['pdict'])
            ]
    return files

# TODO: install cs/LC_MESSAGES... to /usr/local/share/locale (default)
# or to realPrefix/cs/LC_MESS...
trans_prefix = os.path.join(python_site_app_dir, 'po') #gettext._default_localedir
if realPrefix!=sys.prefix:
    trans_prefix = realPrefix

def translations():
    trans = []
    dest = os.path.join(trans_prefix, '%s', 'LC_MESSAGES', '%s.mo')
    for po in glob.glob('po/*.po'):
        lang = os.path.splitext(os.path.basename(po))[0]
        trans.append((dest % (lang, app_name), po))
    return trans

METADATA = {
    'name'              : 'pDict',
    'version'           : version,
    'description'       : 'dictionary search engine',
    'long_description'  : long_desc,
    'author'            : 'Jiří Popek',
    'author_email'      : 'jiri.popek@gmail.com',
    'classifiers'       : filter(None, classifiers.split("\n")),
    'url'               : 'http://www.popek.org/pdict/',
    'keywords'          : 'dictionary',
    'data_files'        : data_files(),
    'translations'      : translations(),
    'trans_install_dir' : trans_prefix,
    'packages'          : ['pdict', 'pdict'],
    'package_dir'       : {'pdict': ''},
    'scripts'           : ['pdict']
}

apply(setup, [], METADATA)
