#!/usr/bin/env python
# -*- coding: utf-8 -*-

# #####################################################################
# Copyright (c) 2005 Jiří Popek <jiri.popek@gmail.com>
#
# name        : pDict
# description : Python dictionary search
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# LICENCE:
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# author : Jiří Popek
# email  : jiri.popek@gmail.com
# date   : 20.3.2005
#
# history:
#        6.8.05 now uses ConfigParser to handle .desc file contents, 
#               dict icon is not necessary now
#       12.7.05 multiple datapaths
# #####################################################################

from globals import *
import os
import sys
from string import split
from sys import exit, argv
from ConfigParser import ConfigParser

def cmpLabels(x, y):
    if x['LABEL']>y['LABEL']: return -1
    elif x['LABEL']<y['LABEL']:return 1
    else: return 0

class Updater(ConfigParser):
    def __init__( self ):
        ConfigParser.__init__(self)

    def _getDirectoryList(self, directory, filter):
        """get list of files matches filter"""
        fileList=[]
        for dir in directory:
            if os.path.exists(dir):
                fl = [os.path.normcase(f) for f in os.listdir(dir)]
                fl = [os.path.join(dir, f) for f in fl \
                        if os.path.splitext(f)[1] in filter]
                fileList.extend(fl)
        
        return fileList

    def getAvailableDictionaries(self):
        """returns dict of available dictonaries"""
        dicts = {}
        # get all .desc files
        fileList = self._getDirectoryList(DATA_PATH, [DESC_FILE_SUFFIX])
        for ds in fileList:
            ds = os.path.splitext(ds)[0]
            if self._allFilesExists(ds):
                # all files exists
                tmp = self.loadConfig(ds)
                bn = os.path.basename(ds)
                if not dicts.has_key(bn):
                    dicts[ds] = (ds, tmp)
        
        ad = {}
        for k,v in dicts.iteritems():
            ad[v[0]] = v[1]
        
        return ad
    
    def _allFilesExists(self, filename):
        """checks if all necessary files exist"""
        files = [filename + DICT_FILE_SUFFIX,
                 filename + INDEX_FILE_SUFFIX]#,
        
        for f in files:
            if not os.path.exists(f):
                return False
        # ok, all necessary files exists
        return True
    
    def loadConfig(self, filename):
        """loads values form .desc (config) file"""
        try:
            self.read(filename)
            items = self.items('global')
        except: # desc file is maybe in old format
            items = self._parseDescFile(filename)
        
        return items
    
    def _parseDescFile(self, filename):
        """parse .desc file"""
        f_vars = open(filename + DESC_FILE_SUFFIX, 'ro')
        vars = f_vars.read()
        vars = split(vars, "\n")
        vars = [split(b, '=') for b in vars]
        vars = dict(vars)
        f_vars.close()
        vars['LABEL'] = unicode(vars['LABEL'], 'utf-8')
        return vars

if __name__ == "__main__":
    updmgr = Updater()
